clear; clc;
warning off;
s = tf('s');

%% Double integrator in continuous time
A  = [0 1; 0 0];
B  = [0; 1];
Cx = eye(size(A));
Dx = zeros(size(Cx,1),size(B,2));
C = [1, 0];
D = 0;

x0 = [1;1];     % Initial condition

%% Discrete form
h1 = 2;
[Phi1,Gamma1] = c2d(A,B,h1);    % Sampled system matrices

h2 = 1;
[Phi2,Gamma2] = c2d(A,B,h2);    % Sampled system matrices

h3 = 0.5;
[Phi3,Gamma3] = c2d(A,B,h3);    % Sampled system matrices

%% Pole placement
% Desired poles in continuous time
omega = 1; zeta = 0.707;
pc = roots([1 2*zeta*omega omega^2]);

% Corresponding desired discrete poles
pd1 = exp(pc*h1);
pd2 = exp(pc*h2);
pd3 = exp(pc*h3);

% Design state feedback
L1 = place(Phi1,Gamma1,pd1);
L = L1; h = h1;
sim('pole_placement.slx');
x1 = x; u1 = u; t1 = t;

L2 = place(Phi2,Gamma2,pd2);
L = L2; h = h2;
sim('pole_placement.slx');
x2 = x; u2 = u; t2 = t;

L3 = place(Phi3,Gamma3,pd3);
L = L3; h = h3;
sim('pole_placement.slx');
x3 = x; u3 = u; t3 = t;

%% Plotting results
figure(1); clf;
subplot(231);
    plot(t1,x1);
    title(sprintf('h=%f',h1));
    ylabel('x');
    xlabel('Time [s]');
subplot(234);
    stairs(t1,u1);
    ylabel('u');
    xlabel('Time [s]');
    ylim([-2,0.5]);

subplot(232);
    plot(t2,x2);
    title(sprintf('h=%f',h2));
    ylabel('x');
    xlabel('Time [s]');
subplot(235);
    stairs(t2,u2);
    ylabel('u');
    xlabel('Time [s]');
    ylim([-2,0.5]);

subplot(233);
    plot(t3,x3);
    title(sprintf('h=%f',h3));
    ylabel('x');
    xlabel('Time [s]');
subplot(236);
    stairs(t3,u3);
    ylabel('u');
    xlabel('Time [s]');
    ylim([-2,0.5]);

%% Deadbeat control
pdb = zeros(size(pc));
Ldb1 = acker(Phi1,Gamma1,pdb);
L = Ldb1; h = h1;
sim('pole_placement.slx');
xdb1 = x; udb1 = u; tdb1 = t;


Ldb2 = acker(Phi2,Gamma2,pdb);
L = Ldb2; h = h2;
sim('pole_placement.slx');
xdb2 = x; udb2 = u; tdb2 = t;

Ldb3 = acker(Phi3,Gamma3,pdb);
L = Ldb3; h = h3;
sim('pole_placement.slx');
xdb3 = x; udb3 = u; tdb3 = t;

%% Plotting results
figure(2); clf;
subplot(231);
    plot(tdb1,xdb1);
    title(sprintf('h=%f',h1));
    ylabel('x');
    xlabel('Time [s]');
subplot(234);
    stairs(tdb1,udb1);
    ylabel('u');
    xlabel('Time [s]');
    ylim([-7.5,6]);

subplot(232);
    plot(tdb2,xdb2);
    title(sprintf('h=%f',h2));
    ylabel('x');
    xlabel('Time [s]');
subplot(235);
    stairs(tdb2,udb2);
    ylabel('u');
    xlabel('Time [s]');
    ylim([-7.5,6]);

subplot(233);
    plot(tdb3,xdb3);
    title(sprintf('h=%f',h3));
    ylabel('x');
    xlabel('Time [s]');
subplot(236);
    stairs(tdb3,udb3);
    ylabel('u');
    xlabel('Time [s]');
    ylim([-7.5,6]);

%% Observer
tend    = 20;
pobs    = pc;
pobs_d1 = exp(pobs*h1);
pobs_d2 = exp(pobs*h2);
pobs_d3 = exp(pobs*h3);

% Design state feedback
K1 = (place(Phi1',C',pobs_d1))';
Phi = Phi1; Gamma = Gamma1;
K = K1; L = L1; h = h1;
sim('observer.slx');
xobs1 = x; xhat1=xhat; uobs1 = u; tobs1 = t;

K2 = (place(Phi2',C',pobs_d2))';
Phi = Phi2; Gamma = Gamma2;
K = K2; L = L2; h = h2;
sim('observer.slx');
xobs2 = x; xhat2=xhat; uobs2 = u; tobs2 = t;

K3 = (place(Phi3',C',pobs_d3))';
Phi = Phi3; Gamma = Gamma3;
K = K3; L = L3; h = h3;
sim('observer.slx');
xobs3 = x; xhat3=xhat; uobs3 = u; tobs3 = t;

%% Plotting results
figure(3); clf;
subplot(231);
hold on;
    plot(tobs1,xobs1);
    plot(tobs1,xhat1,'--');
hold off;
    title(sprintf('h=%f',h1));
    ylabel('x');
    xlabel('Time [s]');
subplot(234);
    stairs(tobs1,uobs1);
    ylabel('u');
    xlabel('Time [s]');
    ylim([-2,2]);

subplot(232);
hold on;
    plot(tobs2,xobs2);
    plot(tobs2,xhat2,'--');
hold off;
    title(sprintf('h=%f',h2));
    ylabel('x');
    xlabel('Time [s]');
subplot(235);
    stairs(tobs2,uobs2);
    ylabel('u');
    xlabel('Time [s]');
    ylim([-2,2]);

subplot(233);
hold on;
    plot(tobs3,xobs3);
    plot(tobs3,xhat3,'--');
hold off;
    title(sprintf('h=%f',h3));
    ylabel('x');
    xlabel('Time [s]');
subplot(236);
    stairs(tobs3,uobs3);
    ylabel('u');
    xlabel('Time [s]');
    ylim([-2,2]);

%% Observer deadbeat
tend    = 20;
pobs_ddb = zeros(size(A,1),1);

% Design state feedback
Kdb1 = (acker(Phi1',C',pobs_ddb))';
Phi = Phi1; Gamma = Gamma1;
K = Kdb1; L = L1; h = h1;
sim('observer.slx');
xobsdb1 = x; xhatdb1=xhat; uobsdb1 = u; tobsdb1 = t;

Kdb2 = (acker(Phi2',C',pobs_ddb))';
Phi = Phi2; Gamma = Gamma2;
K = Kdb2; L = L2; h = h2;
sim('observer.slx');
xobsdb2 = x; xhatdb2=xhat; uobsdb2 = u; tobsdb2 = t;

Kdb3 = (acker(Phi3',C',pobs_ddb))';
Phi = Phi3; Gamma = Gamma3;
K = Kdb3; L = L3; h = h3;
sim('observer.slx');
xobsdb3 = x; xhatdb3=xhat; uobsdb3 = u; tobsdb3 = t;

%% Plotting results
figure(4); clf;
subplot(231);
hold on;
    plot(tobsdb1,xobsdb1);
    plot(tobsdb1,xhatdb1,'--');
hold off;
    title(sprintf('h=%f',h1));
    ylabel('x');
    xlabel('Time [s]');
subplot(234);
    stairs(tobsdb1,uobsdb1);
    ylabel('u');
    xlabel('Time [s]');
    ylim([-4,1]);

subplot(232);
hold on;
    plot(tobsdb2,xobsdb2);
    plot(tobsdb2,xhatdb2,'--');
hold off;
    title(sprintf('h=%f',h2));
    ylabel('x');
    xlabel('Time [s]');
subplot(235);
    stairs(tobsdb2,uobsdb2);
    ylabel('u');
    xlabel('Time [s]');
    ylim([-4,1]);

subplot(233);
hold on;
    plot(tobsdb3,xobsdb3);
    plot(tobsdb3,xhatdb3,'--');
hold off;
    title(sprintf('h=%f',h3));
    ylabel('x');
    xlabel('Time [s]');
subplot(236);
    stairs(tobsdb3,uobsdb3);
    ylabel('u');
    xlabel('Time [s]');
    ylim([-4,1]);
return;

% Design augmented observer
Phie = [Phi Gamma; zeros(1,2) 1];
Ce = [C 0];
Ke = acker(Phie',Ce',[0.75 0.75 0.75])'
